@@ -25,6 +25,7 @@
 #include <QQmlListProperty>
 #include "qmenuitem.h"
 #include "enums.h"
+#include "plasma.h"
 
 class QDeclarativeItem;
 
@@ -82,6 +83,16 @@ class QMenuProxy : public QObject
     Q_PROPERTY(QObject *visualParent READ visualParent WRITE setVisualParent NOTIFY visualParentChanged())
     Q_PROPERTY(DialogStatus::Status status READ status NOTIFY statusChanged)
 
+    /**
+     * The default placement for the menu.
+     */
+    Q_PROPERTY(Plasma::Types::PopupPlacement placement READ placement WRITE setPlacement NOTIFY placementChanged)
+
+    /**
+     * A minimum width for the menu.
+     */
+    Q_PROPERTY(int minimumWidth READ minimumWidth WRITE setMinimumWidth NOTIFY minimumWidthChanged)
+
 public:
     QMenuProxy(QObject *parent = 0);
     ~QMenuProxy();
@@ -97,12 +108,20 @@ public:
     QWindow *transientParent();
     void setTransientParent(QWindow *parent);
 
+    Plasma::Types::PopupPlacement placement() const;
+    void setPlacement(Plasma::Types::PopupPlacement placement);
+
+    int minimumWidth() const;
+    void setMinimumWidth(int width);
 
     /**
      * This opens the menu at position x,y on the given visualParent. By default x and y are set to 0
      */
     Q_INVOKABLE void open(int x = 0, int y = 0);
-    //Q_INVOKABLE void open();
+    /**
+     * This opens the menu at the specified placement relative to the visualParent.
+     */
+    Q_INVOKABLE void openRelative();
     /**
      * This closes the menu
      */
@@ -116,9 +135,13 @@ public:
      */
     Q_INVOKABLE void addMenuItem(const QString &text);
     /**
-     * This adds a menu item of type MenuItem
+     * This adds MenuItem 'item' to the menu before MenuItem 'before'.
+     * If MenuItem 'before' is 0 or does not exist in the menu, 'item'
+     * is appended to the menu instead.
+     * If MenuItem 'item' already exists in the menu, it is removed and
+     * inserted at the new position.
      */
-    Q_INVOKABLE void addMenuItem(QMenuItem *item);
+    Q_INVOKABLE void addMenuItem(QMenuItem *item, QMenuItem *before = nullptr);
     /**
      * This adds a section header with a string used as name for the section
      */
@@ -131,6 +154,8 @@ Q_SIGNALS:
     void statusChanged();
     void visualParentChanged();
     void transientParentChanged();
+    void placementChanged();
+    void minimumWidthChanged();
     void triggered(QMenuItem *item);
     void triggeredIndex(int index);
 
@@ -138,10 +163,14 @@ private Q_SLOTS:
     void itemTriggered(QAction *item);
 
 private:
+    void rebuildMenu();
+    void openInternal(QPoint pos);
+
     QList<QMenuItem *> m_items;
     QMenu *m_menu;
     DialogStatus::Status m_status;
     QWeakPointer<QObject> m_visualParent;
+    Plasma::Types::PopupPlacement m_placement;
 };
 
 #endif //QMENU_PROXY_H
