@@ -36,14 +36,18 @@
 #include <QDebug>
 #include <kfilterdev.h>
 #include <kiconeffect.h>
+#include <KIconLoader>
+#include <KIconTheme>
 
 #include "applet.h"
 #include "package.h"
 #include "theme.h"
+#include "debug_p.h"
 
 namespace Plasma
 {
 
+
 SharedSvgRenderer::SharedSvgRenderer(QObject *parent)
     : QSvgRenderer(parent)
 {
@@ -85,18 +89,19 @@ bool SharedSvgRenderer::load(
             return false;
         }
 
-        QDomNode defs = svg.elementsByTagName("defs").item(0);
+        QDomNode defs = svg.elementsByTagName(QStringLiteral("defs")).item(0);
 
-        for (QDomElement style = defs.firstChildElement("style"); !style.isNull();
-                style = style.nextSiblingElement("style")) {
-            if (style.attribute("id") == "current-color-scheme") {
-                QDomElement colorScheme = svg.createElement("style");
-                colorScheme.setAttribute("type", "text/css");
-                colorScheme.setAttribute("id", "current-color-scheme");
+        const QString STYLE = QStringLiteral("style");
+        for (QDomElement style = defs.firstChildElement(STYLE); !style.isNull();
+                style = style.nextSiblingElement(STYLE)) {
+            if (style.attribute(QStringLiteral("id")) == QLatin1String("current-color-scheme")) {
+                QDomElement colorScheme = svg.createElement(STYLE);
+                colorScheme.setAttribute(QStringLiteral("type"), QStringLiteral("text/css"));
+                colorScheme.setAttribute(QStringLiteral("id"), QStringLiteral("current-color-scheme"));
                 defs.replaceChild(colorScheme, style);
                 colorScheme.appendChild(svg.createCDATASection(styleSheet));
 
-                interestingElements.insert("current-color-scheme", QRect(0, 0, 1, 1));
+                interestingElements.insert(QStringLiteral("current-color-scheme"), QRect(0, 0, 1, 1));
 
                 break;
             }
@@ -110,7 +115,7 @@ bool SharedSvgRenderer::load(
 
     // Search the SVG to find and store all ids that contain size hints.
     const QString contentsAsString(QString::fromLatin1(contents));
-    QRegExp idExpr("id\\s*=\\s*(['\"])(\\d+-\\d+-.*)\\1");
+    QRegExp idExpr(QLatin1String("id\\s*=\\s*(['\"])(\\d+-\\d+-.*)\\1"));
     idExpr.setMinimal(true);
 
     int pos = 0;
@@ -130,7 +135,7 @@ bool SharedSvgRenderer::load(
 
 #define QLSEP QLatin1Char('_')
 #define CACHE_ID_WITH_SIZE(size, id, devicePixelRatio) QString::number(int(size.width())) % QLSEP % QString::number(int(size.height())) % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
-#define CACHE_ID_NATURAL_SIZE(id, devicePixelRatio) QLatin1Literal("Natural") % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
+#define CACHE_ID_NATURAL_SIZE(id, devicePixelRatio) QLatin1String("Natural") % QLSEP % id % QLSEP % QLSEP % QString::number(int(devicePixelRatio))
 
 SvgPrivate::SvgPrivate(Svg *svg)
     : q(svg),
@@ -142,6 +147,7 @@ SvgPrivate::SvgPrivate(Svg *svg)
       scaleFactor(1.0),
       multipleImages(false),
       themed(false),
+      useSystemColors(false),
       fromCurrentTheme(false),
       applyColors(false),
       usesColors(false),
@@ -156,7 +162,7 @@ SvgPrivate::~SvgPrivate()
 }
 
 //This function is meant for the rects cache
-QString SvgPrivate::cacheId(const QString &elementId)
+QString SvgPrivate::cacheId(const QString &elementId) const
 {
     if (size.isValid() && size != naturalSize) {
         return CACHE_ID_WITH_SIZE(size, elementId, devicePixelRatio);
@@ -166,7 +172,7 @@ QString SvgPrivate::cacheId(const QString &elementId)
 }
 
 //This function is meant for the pixmap cache
-QString SvgPrivate::cachePath(const QString &path, const QSize &size)
+QString SvgPrivate::cachePath(const QString &path, const QSize &size) const
 {
     return CACHE_ID_WITH_SIZE(size, path, devicePixelRatio) % QLSEP % QString::number(colorGroup);
 }
@@ -174,12 +180,19 @@ QString SvgPrivate::cachePath(const QString &path, const QSize &size)
 bool SvgPrivate::setImagePath(const QString &imagePath)
 {
     QString actualPath = imagePath;
-    if (imagePath.startsWith("file://")) {
+    if (imagePath.startsWith(QLatin1String("file://"))) {
         //length of file://
         actualPath = actualPath.mid(7);
     }
-    const bool isThemed = !QDir::isAbsolutePath(actualPath);
 
+    bool isThemed = !QDir::isAbsolutePath(actualPath);
+    bool inIconTheme = false;
+
+    //an absolute path.. let's try if this actually an *icon* theme
+    if (!isThemed) {
+        const auto *iconTheme = KIconLoader::global()->theme();
+        isThemed = inIconTheme = iconTheme && actualPath.startsWith(iconTheme->dir());
+    }
     // lets check to see if we're already set to this file
     if (isThemed == themed &&
             ((themed && themePath == actualPath) ||
@@ -212,7 +225,11 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
         emit q->fromCurrentThemeChanged(fromCurrentTheme);
     }
 
-    if (themed) {
+    if (inIconTheme) {
+        themePath = actualPath;
+        path = actualPath;
+        QObject::connect(actualTheme(), SIGNAL(themeChanged()), q, SLOT(themeChanged()));
+    } else if (themed) {
         themePath = actualPath;
         path = actualTheme()->imagePath(themePath);
         themeFailed = path.isEmpty();
@@ -222,7 +239,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
         path = actualPath;
     } else {
 #ifndef NDEBUG
-        // qDebug() << "file '" << path << "' does not exist!";
+        // qCDebug(LOG_PLASMA) << "file '" << path << "' does not exist!";
 #endif
     }
 
@@ -234,14 +251,14 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     if ((themed && QFile::exists(path)) || QFile::exists(actualPath)) {
         QRectF rect;
 
-        if (cacheAndColorsTheme()->findInRectsCache(path, QString("_Natural_%1").arg(scaleFactor), rect)) {
+        if (cacheAndColorsTheme()->findInRectsCache(path, QStringLiteral("_Natural_%1").arg(scaleFactor), rect)) {
             naturalSize = rect.size();
         } else {
             createRenderer();
             naturalSize = renderer->defaultSize() * scaleFactor;
-            //qDebug() << "natural size for" << path << "from renderer is" << naturalSize;
-            cacheAndColorsTheme()->insertIntoRectsCache(path, QString("_Natural_%1").arg(scaleFactor), QRectF(QPointF(0, 0), naturalSize));
-            //qDebug() << "natural size for" << path << "from cache is" << naturalSize;
+            //qCDebug(LOG_PLASMA) << "natural size for" << path << "from renderer is" << naturalSize;
+            cacheAndColorsTheme()->insertIntoRectsCache(path, QStringLiteral("_Natural_%1").arg(scaleFactor), QRectF(QPointF(0, 0), naturalSize));
+            //qCDebug(LOG_PLASMA) << "natural size for" << path << "from cache is" << naturalSize;
         }
     }
 
@@ -268,13 +285,13 @@ Theme *SvgPrivate::actualTheme()
 
 Theme *SvgPrivate::cacheAndColorsTheme()
 {
-    if (themed) {
+    if (themed || !useSystemColors) {
         return actualTheme();
     } else {
         // use a separate cache source for unthemed svg's
         if (!s_systemColorsCache) {
             //FIXME: reference count this, so that it is deleted when no longer in use
-            s_systemColorsCache = new Plasma::Theme("internal-system-colors");
+            s_systemColorsCache = new Plasma::Theme(QStringLiteral("internal-system-colors"));
         }
 
         return s_systemColorsCache.data();
@@ -317,7 +334,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
         if (!elementSizeHints.isEmpty()) {
             QSize bestFit(-1, -1);
 
-            Q_FOREACH (const QSize &hint, elementSizeHints) {
+            Q_FOREACH (QSize hint, elementSizeHints) {
 
                 if (hint.width() >= s.width() * ratio && hint.height() >= s.height() * ratio &&
                         (!bestFit.isValid() ||
@@ -327,8 +344,8 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
             }
 
             if (bestFit.isValid()) {
-                actualElementId = QString::number(bestFit.width()) % "-" %
-                                  QString::number(bestFit.height()) % "-" % elementId;
+                actualElementId = QString::number(bestFit.width()) % '-' %
+                                  QString::number(bestFit.height()) % '-' % elementId;
             }
         }
     }
@@ -353,18 +370,18 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
         id.append(actualElementId);
     }
 
-    //qDebug() << "id is " << id;
+    //qCDebug(LOG_PLASMA) << "id is " << id;
 
     QPixmap p;
     if (cacheRendering && cacheAndColorsTheme()->findInCache(id, p, lastModified)) {
         p.setDevicePixelRatio(ratio);
-        //qDebug() << "found cached version of " << id << p.size();
+        //qCDebug(LOG_PLASMA) << "found cached version of " << id << p.size();
         return p;
     }
 
-    //qDebug() << "didn't find cached version of " << id << ", so re-rendering";
+    //qCDebug(LOG_PLASMA) << "didn't find cached version of " << id << ", so re-rendering";
 
-    //qDebug() << "size for " << actualElementId << " is " << s;
+    //qCDebug(LOG_PLASMA) << "size for " << actualElementId << " is " << s;
     // we have to re-render this puppy
 
     createRenderer();
@@ -407,7 +424,7 @@ void SvgPrivate::createRenderer()
         return;
     }
 
-    //qDebug() << kBacktrace();
+    //qCDebug(LOG_PLASMA) << kBacktrace();
     if (themed && path.isEmpty() && !themeFailed) {
         Applet *applet = qobject_cast<Applet *>(q->parent());
         //FIXME: this maybe could be more efficient if we knew if the package was empty, e.g. for
@@ -415,10 +432,10 @@ void SvgPrivate::createRenderer()
         //for.
         if (applet && applet->kPackage().isValid()) {
             const KPackage::Package package = applet->kPackage();
-            path = package.filePath("images", themePath + ".svg");
+            path = package.filePath("images", themePath + QLatin1String(".svg"));
 
             if (path.isEmpty()) {
-                path = package.filePath("images", themePath + ".svgz");
+                path = package.filePath("images", themePath + QLatin1String(".svgz"));
             }
         }
 
@@ -426,14 +443,14 @@ void SvgPrivate::createRenderer()
             path = actualTheme()->imagePath(themePath);
             themeFailed = path.isEmpty();
             if (themeFailed) {
-                qWarning() << "No image path found for" << themePath;
+                qCWarning(LOG_PLASMA) << "No image path found for" << themePath;
             }
         }
     }
 
-    //qDebug() << "********************************";
-    //qDebug() << "FAIL! **************************";
-    //qDebug() << path << "**";
+    //qCDebug(LOG_PLASMA) << "********************************";
+    //qCDebug(LOG_PLASMA) << "FAIL! **************************";
+    //qCDebug(LOG_PLASMA) << path << "**";
 
     QString styleSheet = cacheAndColorsTheme()->d->svgStyleSheet(colorGroup);
     styleCrc = qChecksum(styleSheet.toUtf8(), styleSheet.size());
@@ -441,7 +458,7 @@ void SvgPrivate::createRenderer()
     QHash<QString, SharedSvgRenderer::Ptr>::const_iterator it = s_renderers.constFind(styleCrc + path);
 
     if (it != s_renderers.constEnd()) {
-        //qDebug() << "gots us an existing one!";
+        //qCDebug(LOG_PLASMA) << "gots us an existing one!";
         renderer = it.value();
     } else {
         if (path.isEmpty()) {
@@ -563,10 +580,10 @@ QMatrix SvgPrivate::matrixForElement(const QString &elementId)
 
 void SvgPrivate::checkColorHints()
 {
-    if (elementRect("hint-apply-color-scheme").isValid()) {
+    if (elementRect(QStringLiteral("hint-apply-color-scheme")).isValid()) {
         applyColors = true;
         usesColors = true;
-    } else if (elementRect("current-color-scheme").isValid()) {
+    } else if (elementRect(QStringLiteral("current-color-scheme")).isValid()) {
         applyColors = false;
         usesColors = true;
     } else {
@@ -576,22 +593,17 @@ void SvgPrivate::checkColorHints()
 
     // check to see if we are using colors, but the theme isn't being used or isn't providing
     // a colorscheme
-    if (qApp) {
+    if (qGuiApp) {
         if (usesColors && (!themed || !actualTheme()->colorScheme())) {
-            qApp->installEventFilter(q);
+            QObject::connect(actualTheme()->d, SIGNAL(applicationPaletteChange()), q, SLOT(colorsChanged()));
         } else {
-            qApp->removeEventFilter(q);
+            QObject::disconnect(actualTheme()->d, SIGNAL(applicationPaletteChange()), q, SLOT(colorsChanged()));
         }
     }
 }
 
 bool Svg::eventFilter(QObject *watched, QEvent *event)
 {
-    if (watched == QCoreApplication::instance()) {
-        if (event->type() == QEvent::ApplicationPaletteChange) {
-            d->colorsChanged();
-        }
-    }
     return QObject::eventFilter(watched, event);
 }
 
@@ -641,7 +653,7 @@ QRectF SvgPrivate::makeUniform(const QRectF &orig, const QRectF &dst)
         res.setHeight(res.height() + offset);
     }
 
-    //qDebug()<<"Aligning Rects, origin:"<<orig<<"destination:"<<dst<<"result:"<<res;
+    //qCDebug(LOG_PLASMA)<<"Aligning Rects, origin:"<<orig<<"destination:"<<dst<<"result:"<<res;
     return res;
 }
 
@@ -662,7 +674,7 @@ void SvgPrivate::themeChanged()
     setImagePath(currentPath);
     q->resize();
 
-    //qDebug() << themePath << ">>>>>>>>>>>>>>>>>> theme changed";
+    //qCDebug(LOG_PLASMA) << themePath << ">>>>>>>>>>>>>>>>>> theme changed";
     emit q->repaintNeeded();
 }
 
@@ -673,17 +685,7 @@ void SvgPrivate::colorsChanged()
     }
 
     eraseRenderer();
-    //qDebug() << "repaint needed from colorsChanged";
-
-    // in the case the theme follows the desktop settings, refetch the colorschemes
-    // and discard the svg pixmap cache
-    if (!theme.data()->d->colors) {
-        KSharedConfig::openConfig()->reparseConfiguration();
-        theme.data()->d->colorScheme = KColorScheme(QPalette::Active, KColorScheme::Window);
-        theme.data()->d->buttonColorScheme = KColorScheme(QPalette::Active, KColorScheme::Button);
-        theme.data()->d->viewColorScheme = KColorScheme(QPalette::Active, KColorScheme::View);
-        theme.data()->d->discardCache(PixmapCache | SvgElementsCache);
-    }
+    qCDebug(LOG_PLASMA) << "repaint needed from colorsChanged";
 
     emit q->repaintNeeded();
 }
@@ -739,7 +741,7 @@ void Svg::setScaleFactor(qreal ratio)
     //not resize() because we want to do it unconditionally
     QRectF rect;
 
-    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QString("_Natural_%1").arg(d->scaleFactor), rect)) {
+    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QStringLiteral("_Natural_%1").arg(d->scaleFactor), rect)) {
         d->naturalSize = rect.size();
     } else {
         d->createRenderer();
@@ -890,7 +892,7 @@ bool Svg::isValid() const
 
     //try very hard to avoid creation of a parser
     QRectF rect;
-    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QString("_Natural_%1").arg(d->scaleFactor), rect)) {
+    if (d->cacheAndColorsTheme()->findInRectsCache(d->path, QStringLiteral("_Natural_%1").arg(d->scaleFactor), rect)) {
         return true;
     }
 
@@ -915,7 +917,7 @@ bool Svg::containsMultipleImages() const
 void Svg::setImagePath(const QString &svgFilePath)
 {
     if (d->setImagePath(svgFilePath)) {
-        //qDebug() << "repaintNeeded";
+        //qCDebug(LOG_PLASMA) << "repaintNeeded";
         emit repaintNeeded();
     }
 }
@@ -940,6 +942,21 @@ bool Svg::fromCurrentTheme() const
     return d->fromCurrentTheme;
 }
 
+void Svg::setUseSystemColors(bool system)
+{
+    if (d->useSystemColors == system) {
+        return;
+    }
+
+    d->useSystemColors = system;
+    emit repaintNeeded();
+}
+
+bool Svg::useSystemColors() const
+{
+    return d->useSystemColors;
+}
+
 void Svg::setTheme(Plasma::Theme *theme)
 {
     if (!theme || theme == d->theme.data()) {
@@ -962,4 +979,5 @@ Theme *Svg::theme() const
 
 } // Plasma namespace
 
+#include "private/moc_svg_p.cpp"
 #include "moc_svg.cpp"
